"""Contains some shared types for properties"""

from collections.abc import (
    Mapping,
    MutableMapping,
)
from http import HTTPStatus
from typing import (
    IO,
    BinaryIO,
    Generic,
    Literal,
    Optional,
    TypeVar,
    Union,
)

from attrs import define


class Unset:
    def __bool__(self) -> Literal[False]:
        return False


UNSET: Unset = Unset()

# The types that `httpx.Client(files=)` can accept, copied from that library.
FileContent = Union[IO[bytes], bytes, str]
FileTypes = Union[
    # (filename, file (or bytes), content_type)
    tuple[Optional[str], FileContent, Optional[str]],
    # (filename, file (or bytes), content_type, headers)
    tuple[Optional[str], FileContent, Optional[str], Mapping[str, str]],
]
RequestFiles = list[tuple[str, FileTypes]]


@define
class File:
    """Contains information for file uploads"""

    payload: BinaryIO
    file_name: Optional[str] = None
    mime_type: Optional[str] = None

    def to_tuple(self) -> FileTypes:
        """Return a tuple representation that httpx will accept for multipart/form-data"""
        return self.file_name, self.payload, self.mime_type


T = TypeVar("T")


@define
class Response(Generic[T]):
    """A response from an endpoint"""

    status_code: HTTPStatus
    content: bytes
    headers: MutableMapping[str, str]
    parsed: Optional[T]


__all__ = ["UNSET", "File", "FileTypes", "RequestFiles", "Response", "Unset"]



from collections.abc import (
    Generator,
    Mapping,
)
from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Optional,
    TextIO,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="ClusterSettingsUpdate")


@_attrs_define
class ClusterSettingsUpdate:
    """
    Attributes:
        offload_enabled (Union[Unset, bool]):
        offload_timeout_min (Union[Unset, int]):
    """

    offload_enabled: Union[Unset, bool] = UNSET
    offload_timeout_min: Union[Unset, int] = UNSET

    def to_dict(self) -> dict[str, Any]:
        offload_enabled = self.offload_enabled

        offload_timeout_min = self.offload_timeout_min

        field_dict: dict[str, Any] = {}

        field_dict.update({})
        if offload_enabled is not UNSET:
            field_dict["offloadEnabled"] = offload_enabled
        if offload_timeout_min is not UNSET:
            field_dict["offloadTimeoutMin"] = offload_timeout_min

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        offload_enabled = d.pop("offloadEnabled", UNSET)

        offload_timeout_min = d.pop("offloadTimeoutMin", UNSET)

        cluster_settings_update = cls(
            offload_enabled=offload_enabled,
            offload_timeout_min=offload_timeout_min,
        )

        return cluster_settings_update



from collections.abc import (
    Generator,
    Mapping,
)
from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Optional,
    TextIO,
    TypeVar,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="DlhcActivateStatus")


@_attrs_define
class DlhcActivateStatus:
    """
    Attributes:
        status (str):
        created_at (str):
    """

    status: str
    created_at: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        status = self.status

        created_at = self.created_at

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
                "created_at": created_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        status = d.pop("status")

        created_at = d.pop("created_at")

        dlhc_activate_status = cls(
            status=status,
            created_at=created_at,
        )

        dlhc_activate_status.additional_properties = d
        return dlhc_activate_status

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from collections.abc import (
    Generator,
    Mapping,
)
from typing import (
    TYPE_CHECKING,
    Any,
    BinaryIO,
    Optional,
    TextIO,
    TypeVar,
    Union,
)

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import (
    UNSET,
    Unset,
)

T = TypeVar("T", bound="ExtensionParameterDefinitions")


@_attrs_define
class ExtensionParameterDefinitions:
    """
    Attributes:
        id (str):
        name (str):
        raw_definition (Union[Unset, Any]):
    """

    id: str
    name: str
    raw_definition: Union[Unset, Any] = UNSET

    def to_dict(self) -> dict[str, Any]:
        id = self.id

        name = self.name

        raw_definition = self.raw_definition

        field_dict: dict[str, Any] = {}

        field_dict.update(
            {
                "id": id,
                "name": name,
            }
        )
        if raw_definition is not UNSET:
            field_dict["rawDefinition"] = raw_definition

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        id = d.pop("id")

        name = d.pop("name")

        raw_definition = d.pop("rawDefinition", UNSET)

        extension_parameter_definitions = cls(
            id=id,
            name=name,
            raw_definition=raw_definition,
        )

        return extension_parameter_definitions



from http import HTTPStatus
from typing import (
    Any,
    Optional,
    Union,
    cast,
)

import httpx

from ... import errors
from ...client import (
    AuthenticatedClient,
    Client,
)
from ...models.get_usage_type import GetUsageType
from ...models.usage import Usage
from ...types import (
    UNSET,
    Response,
    Unset,
)


def _get_kwargs(
    account_id: str,
    *,
    year_month: Union[Unset, str] = UNSET,
    type_: Union[Unset, GetUsageType] = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    params["yearMonth"] = year_month

    json_type_: Union[Unset, str] = UNSET
    if not isinstance(type_, Unset):
        json_type_ = type_.value

    params["type"] = json_type_

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/api/v1/accounts/{account_id}/usage",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Usage]:
    if response.status_code == 200:
        response_200 = Usage.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Usage]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    account_id: str,
    *,
    client: AuthenticatedClient,
    year_month: Union[Unset, str] = UNSET,
    type_: Union[Unset, GetUsageType] = UNSET,
) -> Response[Usage]:
    """
    Args:
        account_id (str):
        year_month (Union[Unset, str]):
        type_ (Union[Unset, GetUsageType]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Usage]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        year_month=year_month,
        type_=type_,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    account_id: str,
    *,
    client: AuthenticatedClient,
    year_month: Union[Unset, str] = UNSET,
    type_: Union[Unset, GetUsageType] = UNSET,
) -> Optional[Usage]:
    """
    Args:
        account_id (str):
        year_month (Union[Unset, str]):
        type_ (Union[Unset, GetUsageType]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Usage
    """

    return sync_detailed(
        account_id=account_id,
        client=client,
        year_month=year_month,
        type_=type_,
    ).parsed


async def asyncio_detailed(
    account_id: str,
    *,
    client: AuthenticatedClient,
    year_month: Union[Unset, str] = UNSET,
    type_: Union[Unset, GetUsageType] = UNSET,
) -> Response[Usage]:
    """
    Args:
        account_id (str):
        year_month (Union[Unset, str]):
        type_ (Union[Unset, GetUsageType]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Usage]
    """

    kwargs = _get_kwargs(
        account_id=account_id,
        year_month=year_month,
        type_=type_,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    account_id: str,
    *,
    client: AuthenticatedClient,
    year_month: Union[Unset, str] = UNSET,
    type_: Union[Unset, GetUsageType] = UNSET,
) -> Optional[Usage]:
    """
    Args:
        account_id (str):
        year_month (Union[Unset, str]):
        type_ (Union[Unset, GetUsageType]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Usage
    """

    return (
        await asyncio_detailed(
            account_id=account_id,
            client=client,
            year_month=year_month,
            type_=type_,
        )
    ).parsed
